@@ -186,12 +186,14 @@ public class JVMStabilityInspectorCorruptSSTableExceptionTest extends TestBaseIm
             super(delegate);
         }
 
-        public UnfilteredRowIterator iterator(DecoratedKey key, Slices slices, ColumnFilter selectedColumns, boolean reversed, boolean isForThrift, SSTableReadsListener listener)
+        @Override
+        public UnfilteredRowIterator iterator(DecoratedKey key, Slices slices, ColumnFilter selectedColumns, boolean reversed, SSTableReadsListener listener)
         {
             throw throwCorrupted();
         }
 
-        public UnfilteredRowIterator iterator(FileDataInput file, DecoratedKey key, RowIndexEntry indexEntry, Slices slices, ColumnFilter selectedColumns, boolean reversed, boolean isForThrift)
+        @Override
+        public UnfilteredRowIterator iterator(FileDataInput file, DecoratedKey key, RowIndexEntry indexEntry, Slices slices, ColumnFilter selectedColumns, boolean reversed)
         {
             throw throwCorrupted();
         }
@@ -201,4 +203,4 @@ public class JVMStabilityInspectorCorruptSSTableExceptionTest extends TestBaseIm
             throw new CorruptSSTableException(new IOException("failed to get position"), descriptor.baseFilename());
         }
     }
-}
\ No newline at end of file
+}
